@@ -22,7 +22,9 @@
 
 #include <sys/types.h>
 #include <pwd.h>
+#ifdef HAVE_SHADOW_H
 #include <shadow.h>
+#endif
 
 #include <glib.h>
 #include <gio/gio.h>
@@ -57,8 +59,7 @@ User *         user_new                     (Daemon        *daemon,
 void           user_update_from_pwent       (User          *user,
                                              struct passwd *pwent,
                                              struct spwd   *spent);
-void           user_update_from_keyfile     (User          *user,
-                                             GKeyFile      *keyfile);
+void           user_update_from_cache       (User *user);
 void           user_update_local_account_property (User          *user,
                                                    gboolean       local);
 void           user_update_system_account_property (User          *user,
