@@ -37,52 +37,43 @@
 static gchar *
 get_cmdline_of_pid (GPid pid)
 {
-  gchar *ret = NULL;
-  gchar *filename;
-  gchar *contents;
-  gsize contents_len;
-  GError *error;
-  guint n;
+        gchar *ret;
+        g_autofree gchar *filename = NULL;
+        g_autofree gchar *contents = NULL;
+        gsize contents_len;
+        g_autoptr(GError) error = NULL;
+        guint n;
 
-  filename = g_strdup_printf ("/proc/%d/cmdline", (int) pid);
+        filename = g_strdup_printf ("/proc/%d/cmdline", (int) pid);
 
-  if (!g_file_get_contents (filename,
-                            &contents,
-                            &contents_len,
-                            &error))
-    {
-      g_warning ("Error opening `%s': %s",
-                 filename,
-                 error->message);
-      g_error_free (error);
-      goto out;
-    }
-  /* The kernel uses '\0' to separate arguments - replace those with a space. */
-  for (n = 0; n < contents_len - 1; n++)
-    {
-      if (contents[n] == '\0')
-        contents[n] = ' ';
-    }
+        if (!g_file_get_contents (filename,
+                                  &contents,
+                                  &contents_len,
+                                  &error)) {
+                g_warning ("Error opening `%s': %s",
+                           filename,
+                           error->message);
+                return NULL;
+        }
+        /* The kernel uses '\0' to separate arguments - replace those with a space. */
+        for (n = 0; n < contents_len - 1; n++) {
+                if (contents[n] == '\0')
+                        contents[n] = ' ';
+        }
 
-  ret = g_strdup (contents);
-  g_strstrip (ret);
-
- out:
-  g_free (filename);
-  g_free (contents);
-
-  return ret;
+        ret = g_strdup (contents);
+        g_strstrip (ret);
+        return ret;
 }
 
 static gboolean
 get_caller_pid (GDBusMethodInvocation *context,
                 GPid                  *pid)
 {
-        GVariant      *reply;
-        GError        *error;
-        guint32        pid_as_int;
+        g_autoptr(GVariant) reply = NULL;
+        g_autoptr(GError) error = NULL;
+        guint32 pid_as_int;
 
-        error = NULL;
         reply = g_dbus_connection_call_sync (g_dbus_method_invocation_get_connection (context),
                                              "org.freedesktop.DBus",
                                              "/org/freedesktop/DBus",
@@ -100,16 +91,12 @@ get_caller_pid (GDBusMethodInvocation *context,
                 g_warning ("Could not talk to message bus to find uid of sender %s: %s",
                            g_dbus_method_invocation_get_sender (context),
                            error->message);
-                g_error_free (error);
-
                 return FALSE;
         }
 
         g_variant_get (reply, "(u)", &pid_as_int);
         *pid = pid_as_int;
 
-        g_variant_unref (reply);
-
         return TRUE;
 }
 
@@ -119,7 +106,7 @@ sys_log (GDBusMethodInvocation *context,
                                 ...)
 {
         va_list args;
-        gchar *msg;
+        g_autofree gchar *msg = NULL;
 
         va_start (args, format);
         msg = g_strdup_vprintf (format, args);
@@ -127,11 +114,11 @@ sys_log (GDBusMethodInvocation *context,
 
         if (context) {
                 PolkitSubject *subject;
-                gchar *cmdline = NULL;
-                gchar *id;
+                g_autofree gchar *cmdline = NULL;
+                g_autofree gchar *id = NULL;
                 GPid pid = 0;
                 gint uid = -1;
-                gchar *tmp;
+                g_autofree gchar *tmp = NULL;
 
                 subject = polkit_system_bus_name_new (g_dbus_method_invocation_get_sender (context));
                 id = polkit_subject_to_string (subject);
@@ -160,29 +147,20 @@ sys_log (GDBusMethodInvocation *context,
                 }
 
                 g_free (msg);
-                msg = tmp;
+                msg = g_steal_pointer (&tmp);
 
-                g_free (id);
-                g_free (cmdline);
                 g_object_unref (subject);
         }
 
         syslog (LOG_NOTICE, "%s", msg);
-
-        g_free (msg);
 }
 
 static void
 get_caller_loginuid (GDBusMethodInvocation *context, gchar *loginuid, gint size)
 {
         GPid pid;
-        gint uid;
-        gchar *path;
-        gchar *buf;
-
-        if (!get_caller_uid (context, &uid)) {
-                uid = getuid ();
-        }
+        g_autofree gchar *path = NULL;
+        g_autofree gchar *buf = NULL;
 
         if (get_caller_pid (context, &pid)) {
                 path = g_strdup_printf ("/proc/%d/loginuid", (int) pid);
@@ -192,13 +170,15 @@ get_caller_loginuid (GDBusMethodInvocation *context, gchar *loginuid, gint size)
 
         if (path != NULL && g_file_get_contents (path, &buf, NULL, NULL)) {
                 strncpy (loginuid, buf, size);
-                g_free (buf);
         }
         else {
+                gint uid;
+
+                if (!get_caller_uid (context, &uid)) {
+                        uid = getuid ();
+                }
                 g_snprintf (loginuid, size, "%d", uid);
         }
-
-        g_free (path);
 }
 
 static gboolean
@@ -277,15 +257,85 @@ get_user_groups (const gchar  *user,
         return res;
 }
 
+/**
+ * get_admin_groups:
+ * @admin_gid_out: (out caller-allocates) (optional): return location for the ID
+ *    of the main admin group
+ * @groups_out: (out callee-allocates) (transfer container) (optional) (length=n_groups_out):
+ *    return location for an array of the extra admin group IDs
+ * @n_groups_out: (out caller-allocates) (optional): return location for the
+ *    number of elements in @group_out
+ *
+ * Get the GIDs of the admin groups on the system, as set at configure time for
+ * accountsservice. The main admin group ID (typically for the `sudo` or `wheel`
+ * group) will be returned in @admin_gid_out. Any group IDs for other admin
+ * groups (such as `lpadmin` or `systemd-journal`) will be returned in
+ * @groups_out, which should be freed by the caller using g_free().
+ *
+ * Returns: %TRUE on success, %FALSE if one or more of the groups could not be
+ *    looked up
+ */
+gboolean
+get_admin_groups (gid_t  *admin_gid_out,
+                  gid_t **groups_out,
+                  gsize  *n_groups_out)
+{
+        g_auto(GStrv) extra_admin_groups = NULL;
+        g_autofree gid_t *extra_admin_groups_gids = NULL;
+        gsize n_extra_admin_groups_gids = 0;
+        gsize i;
+        gboolean retval = FALSE;
+        struct group *grp;
+        gid_t admin_gid = 0;
+
+        /* Get the main admin group ID. */
+        grp = getgrnam (ADMIN_GROUP);
+        if (grp == NULL)
+                goto out;
+        admin_gid = grp->gr_gid;
+
+        /* Get the extra admin group IDs. */
+        extra_admin_groups = g_strsplit (EXTRA_ADMIN_GROUPS, ",", 0);
+        n_extra_admin_groups_gids = 0;
+        extra_admin_groups_gids = g_new0 (gid_t, g_strv_length (extra_admin_groups));
+
+        for (i = 0; extra_admin_groups[i] != NULL; i++) {
+                struct group *extra_group;
+                extra_group = getgrnam (extra_admin_groups[i]);
+                if (extra_group == NULL)
+                        goto out;
+                if (extra_group->gr_gid == admin_gid)
+                        continue;
+
+                extra_admin_groups_gids[n_extra_admin_groups_gids++] = extra_group->gr_gid;
+        }
+
+        retval = TRUE;
+
+out:
+        if (!retval) {
+                admin_gid = 0;
+                g_clear_pointer (&extra_admin_groups_gids, g_free);
+                n_extra_admin_groups_gids = 0;
+        }
+
+        if (admin_gid_out != NULL)
+                *admin_gid_out = admin_gid;
+        if (groups_out != NULL)
+                *groups_out = g_steal_pointer (&extra_admin_groups_gids);
+        if (n_groups_out != NULL)
+                *n_groups_out = n_extra_admin_groups_gids;
+
+        return retval;
+}
 
 gboolean
 get_caller_uid (GDBusMethodInvocation *context,
                 gint                  *uid)
 {
-        GVariant      *reply;
-        GError        *error;
+        g_autoptr(GVariant) reply = NULL;
+        g_autoptr(GError) error = NULL;
 
-        error = NULL;
         reply = g_dbus_connection_call_sync (g_dbus_method_invocation_get_connection (context),
                                              "org.freedesktop.DBus",
                                              "/org/freedesktop/DBus",
@@ -303,13 +353,10 @@ get_caller_uid (GDBusMethodInvocation *context,
                 g_warning ("Could not talk to message bus to find uid of sender %s: %s",
                            g_dbus_method_invocation_get_sender (context),
                            error->message);
-                g_error_free (error);
-
                 return FALSE;
         }
 
         g_variant_get (reply, "(u)", uid);
-        g_variant_unref (reply);
 
         return TRUE;
 }
