@@ -75,7 +75,6 @@ wtmp_helper_update_login_frequencies (GHashTable *users)
         struct utmpx *wtmp_entry;
         GHashTableIter iter;
         gpointer key, value;
-        struct passwd *pwent;
         User *user;
         GVariantBuilder *builder, *builder2;
         GList *l;
@@ -128,11 +127,6 @@ wtmp_helper_update_login_frequencies (GHashTable *users)
                         continue;
                 }
 
-                pwent = getpwnam (wtmp_entry->ut_user);
-                if (pwent == NULL) {
-                        continue;
-                }
-
                 if (!g_hash_table_lookup_extended (login_hash,
                                                    wtmp_entry->ut_user,
                                                    &key, &value)) {
